function [Tf, jdet] = PML(data,label)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% demo code of Projection Metric Learning (PML)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Fixed param
n_iter = 10; % number of iteration

% Setting param

num_sample = length(data);
pca_dim = size(data{1},2);
num_class = length(unique(label));
PCARatio = 0.95;


for i = 1 : num_sample
    Pm(:,(i-1)*pca_dim+1:i*pca_dim)= data{i};
end


[P, D] = eig(Pm*Pm');
eig_value_unsort = diag(D);
[eig_value,  eig_index]  = sort(eig_value_unsort,'descend');

sumEig = sum(eig_value);
sumEig = sumEig*PCARatio;
sumNow = 0;
for idx = 1:length(eig_value)
    sumNow = sumNow + eig_value(idx);
    if sumNow >= sumEig
        break;
    end
end
P = fliplr(P); P = P(:,1:idx);

pcaData = cell(num_sample,1);
for i = 1 : num_sample
    pcaData{i}= P'*data{i};
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Iterative optimization
u_dim_max = idx;
T_0 = eye(u_dim_max,u_dim_max)*rand(u_dim_max,u_dim_max);
T = T_0;

for iter=1:n_iter
    fprintf('\n iter %d \n',iter);
    
    % Step-1: Normalize P
    normData = NormalizeP(pcaData,T);
    
    Sw = zeros(u_dim_max,u_dim_max); Sb = zeros(u_dim_max,u_dim_max);
    
    nw = 0; nb=0;
    % Within-class sets
    
    for i = 1 : num_class
        ind = find(label==i);
        for j = 1 : length(ind)
            V1 = normData{ind(j)};
            for k = j+1: length(ind)
                V2 = normData{ind(k)};
                can1 = V1*V1'; can2 = V2*V2';
                dtemp = can1 - can2; Sdtemp = dtemp*dtemp';
                Sw = Sw + Sdtemp; nw = nw+1;
            end
        end
    end
    
    for i = 1 : num_class
        ind = find(label==i);
        dif = find(label~=i);
        for j = 1 : length(ind)
            V1 = normData{ind(j)};
            for k = 1: length(dif)
                V2 = normData{dif(k)};
                can1 = V1*V1'; can2 = V2*V2';
                dtemp = can1 - can2; Sdtemp = dtemp*dtemp';
                Sb = Sb + Sdtemp; nb = nb+1;
            end
        end
    end
    
    
    % Step-3 : Compute the transformation matrix T
    Sdw = Sw/(nw); Sdb = Sb/(nb);
    Sd = inv(Sdw)*Sdb;
    
    fprintf('log(det) :'); jdet(iter) = log(det(Sd))
    
    if(iter>1)
        if(jdet(iter)<jdet(iter-1)) break; end
    end
    
    
    %Riemannian Conjugate Gradient (RCG)
    alpha = 0.2;
    spd = sympositivedefinitefactory(u_dim_max);
    problem.M = spd;
    problem.cost = @(W)   trace(W'*(Sdb-alpha*Sdw)*W);
    problem.grad = @(W)   spd.egrad2rgrad(W, 2*(Sdb-alpha*Sdw)*W);
    options = [];
    options.maxiter = 100;
    [W cost info] = conjugategradient(problem, [], options);
    T  = sqrtm(W);
end % End of loop 'iter'

% Get final transformation
Tf = P*T;
